
import React, { ReactNode, useEffect, useState, Children, isValidElement, ReactElement } from 'react';
import Footer from './Footer';
import Navbar from './Navbar';
import ScrollRestoration from '../routing/ScrollRestoration';
import { ScrollArea } from '@/components/ui/scroll-area';
import { AppSidebar } from './AppSidebar';
import ReturnToTopButton from '@/components/ui/ReturnToTopButton';
import { useIsMobile } from '@/hooks/use-mobile';

interface PageLayoutProps {
  children: ReactNode;
  withSidebar?: boolean;
  withoutHeader?: boolean;
  withoutFooter?: boolean;
  className?: string;
  containerStyle?: React.CSSProperties;
  scrollToTop?: boolean;
}

// Check if a component contains a specific component type
const hasComponentOfType = (children: React.ReactNode, componentType: React.ComponentType<any>) => {
  let hasComponent = false;
  
  const checkForComponent = (child: React.ReactNode) => {
    if (!isValidElement(child)) return;
    
    // Check element type
    if (child.type === componentType) {
      hasComponent = true;
      return;
    }
    
    // Recursively check children
    if (child.props && typeof child.props === 'object' && 'children' in child.props) {
      Children.forEach(child.props.children, checkForComponent);
    }
  };
  
  Children.forEach(children, checkForComponent);
  return hasComponent;
};

// Process children to prevent duplicate headers and footers
const processChildren = (children: React.ReactNode): React.ReactNode => {
  // Convert children to array for easier handling
  const childrenArray = Children.toArray(children);
  
  // Check for Home component to avoid nesting with Index
  const hasHomeComponent = childrenArray.some(
    child => isValidElement(child) && 
    ((child.type as any)?.displayName === 'Home' || 
     (child.type as any)?.name === 'Home')
  );
  
  if (hasHomeComponent) {
    console.log("PageLayout: Detected Home component, preventing duplication");
    // Extract children from Home if present
    const extractedChildren = childrenArray.map((child) => {
      if (isValidElement(child) && 
         ((child.type as any)?.displayName === 'Home' || 
          (child.type as any)?.name === 'Home')) {
        // Return the child's children directly with proper type assertion
        return (child as ReactElement<{children?: ReactNode}>).props.children || null;
      }
      return child;
    });
    
    return processChildren(extractedChildren);
  }
  
  // Check for nested PageLayout components
  const hasPageLayout = childrenArray.some(child => 
    isValidElement(child) && child.type === PageLayout
  );
  
  if (hasPageLayout) {
    console.log("PageLayout: Detected nested PageLayout, preventing duplication");
    // Extract children from nested PageLayouts
    const extractedChildren = childrenArray.flatMap((child) => {
      if (isValidElement(child) && child.type === PageLayout) {
        // Only return children of nested PageLayout
        const layoutChild = child as ReactElement<PageLayoutProps>;
        return Children.toArray(layoutChild.props.children);
      }
      return [child];
    });
    
    return extractedChildren;
  }
  
  return childrenArray;
};

const PageLayout: React.FC<PageLayoutProps> = ({
  children,
  withSidebar = false,
  withoutHeader = false,
  withoutFooter = false,
  className = '',
  containerStyle = {},
  scrollToTop = true,
}) => {
  const isMobile = useIsMobile();
  const [shouldRenderHeader, setShouldRenderHeader] = useState(!withoutHeader);
  const [shouldRenderFooter, setShouldRenderFooter] = useState(!withoutFooter);
  const [processedChildren, setProcessedChildren] = useState<React.ReactNode>(children);
  
  // Process children and check for headers/footers on mount and when children change
  useEffect(() => {
    // Process children to avoid nested duplicate components
    const processed = processChildren(children);
    setProcessedChildren(processed);
    
    // Check if processed children already contain header and footer
    const hasNavbar = hasComponentOfType(processed, Navbar);
    const hasFooter = hasComponentOfType(processed, Footer);
    
    setShouldRenderHeader(!withoutHeader && !hasNavbar);
    setShouldRenderFooter(!withoutFooter && !hasFooter);
  }, [children, withoutHeader, withoutFooter]);

  const mainContent = (
    <>
      {shouldRenderHeader && <Navbar />}
      <main className={`min-h-[calc(100vh-4rem)] flex flex-col ${className}`} style={containerStyle}>
        <div className="flex-grow">
          {processedChildren}
        </div>
        {shouldRenderFooter && <Footer />}
      </main>
      {scrollToTop && <ReturnToTopButton />}
    </>
  );

  return (
    <>
      <ScrollRestoration />
      {withSidebar ? (
        <div className={`flex ${isMobile ? 'flex-col' : 'flex-row'}`}>
          <AppSidebar />
          <div className={`flex-grow ${isMobile ? 'w-full' : 'pl-64'}`}>
            <ScrollArea className={`h-screen ${isMobile ? 'h-[calc(100vh-64px)]' : ''}`}>
              {mainContent}
            </ScrollArea>
          </div>
        </div>
      ) : (
        mainContent
      )}
    </>
  );
};

export default PageLayout;
